@@ -1,6 +1,5 @@
 import asyncio
 import logging
-import socket
 from abc import abstractmethod, abstractproperty
 from dataclasses import dataclass, field
 from timeit import default_timer
@@ -19,43 +18,13 @@ from faust.types.models import ModelT
 
 from delphai_utils.config import get_config
 from delphai_utils.patches import aiokafka_patch
-
+from delphai_utils.utils import find_free_port
 
 logger = logging.getLogger(__name__)
 
 aiokafka_patch()
 
 
-def is_port_free(host, port):
-    """
-    determine whether `host` has the `port` free
-
-    From: https://www.thepythoncode.com/article/make-port-scanner-python
-    """
-    s = socket.socket()
-    try:
-        s.connect((host, port))
-    except Exception:
-        return True
-    else:
-        return False
-
-
-class NoPortFoundError(Exception):
-    ...
-
-
-def find_free_port(start: int, host="127.0.0.1", num_tries=4) -> int:
-    for port in range(start, start + num_tries):
-        if is_port_free(host, port):
-            return port
-        else:
-            logger.info(f"Port {port} already in use.")
-    message = f"No free port found in range [{start}, {start + num_tries - 1}]"
-    logger.error(message)
-    raise NoPortFoundError(message)
-
-
 @dataclass
 class Step:
     name: str
