@@ -1,10 +1,11 @@
+import asyncio
 import collections.abc
 import json
 import logging
 import os
 from base64 import b64encode
 from time import perf_counter
-from typing import Dict, Iterable, List, Union
+from typing import Dict, Iterable, List, Optional, Union
 from urllib.parse import urlparse
 
 from google.api.http_pb2 import HttpRule
@@ -18,7 +19,7 @@ from google.protobuf.descriptor import (
 from google.protobuf.descriptor_pb2 import MethodOptions
 from google.protobuf.json_format import MessageToDict
 from google.protobuf.message import Message
-from grpc import Channel, StatusCode
+from grpc import Channel, Server, StatusCode
 from grpc.experimental.aio import AioRpcError, insecure_channel
 from hypercorn.asyncio import serve
 from hypercorn.config import Config
@@ -29,10 +30,10 @@ from starlette.middleware.base import BaseHTTPMiddleware
 from starlette.middleware.cors import CORSMiddleware
 from starlette.requests import Request
 from starlette.responses import JSONResponse
-from starlette_prometheus import PrometheusMiddleware, metrics
+from starlette_prometheus import PrometheusMiddleware
 
 from delphai_utils.keycloak import PublicKeyFetchError, decode_token
-from delphai_utils.utils import generate_default_message
+from delphai_utils.utils import find_free_port, generate_default_message
 
 logger = logging.getLogger(__name__)
 
@@ -204,82 +205,82 @@ def get_http_handlers(
     return method_get_handler, request_handler
 
 
-async def start_gateway(
-    descriptors: Union[FileDescriptor, Iterable[FileDescriptor]],
+def start_gateway(
+    server: Server,
     grpc_port: int,
-    http_port: int = 7070,
+    http_port: Optional[int] = None,
+    app: Optional[Starlette] = None,
 ):
-    if not isinstance(descriptors, collections.abc.Iterable):
-        descriptors = [descriptors]
-    try:
-        logger.info("starting gateway...")
-        debug = (
-            "DELPHAI_ENVIRONMENT" in os.environ
-            and os.environ["DELPHAI_ENVIRONMENT"] == "development"
-        )
+    if not app:
+        debug = os.environ.get("DELPHAI_ENVIRONMENT") == "development"
         app = Starlette(debug=debug)
-        app.add_exception_handler(HTTPException, http_exception)
-        app.add_middleware(AccessLogMiddleware)
-        app.add_middleware(PrometheusMiddleware)
-        app.add_middleware(
-            CORSMiddleware,
-            allow_origins=["*"],
-            allow_methods=["*"],
-            allow_headers=["*"],
-            allow_credentials=True,
+
+    app.add_exception_handler(HTTPException, http_exception)
+    app.add_middleware(AccessLogMiddleware)
+    app.add_middleware(PrometheusMiddleware)
+    app.add_middleware(
+        CORSMiddleware,
+        allow_origins=["*"],
+        allow_methods=["*"],
+        allow_headers=["*"],
+        allow_credentials=True,
+    )
+
+    from .grpc_server import GRPC_OPTIONS
+
+    channel = insecure_channel(f"localhost:{grpc_port}", options=GRPC_OPTIONS)
+
+    if not hasattr(server, "descriptors"):
+        raise RuntimeError(
+            "Server instance does not include the proto file descriptors. Make sure you instantiate it with 'create_grpc_server'"
         )
-        app.add_exception_handler(HTTPException, http_exception)
-        app.add_route("/metrics/", metrics)
-        max_length = 512 * 1024 * 1024
-        client_options = [
-            ("grpc.max_send_message_length", max_length),
-            ("grpc.max_receive_message_length", max_length),
-        ]
-        channel = insecure_channel(f"localhost:{grpc_port}", options=client_options)
-        config = Config()
-        config.bind = [f"0.0.0.0:{http_port}"]
-        for descriptor in descriptors:
-            for service_name in descriptor.services_by_name:
-                service_handler: ServiceDescriptor = descriptor.services_by_name[
-                    service_name
-                ]
-                if service_handler.full_name.startswith("grpc."):
-                    logger.info(f"skipping service {service_handler.name}")
-                else:
-                    logger.info(f"processing service {service_handler.name}")
-                    for method_name in service_handler.methods_by_name.keys():
-                        method_get_handler, request_handler = get_http_handlers(
-                            descriptor, service_handler.name, method_name, channel
-                        )
-                        service_name = method_name[1:].split("/")[0].split(".")[-1]
-                        app.add_route(
-                            f"/{service_handler.full_name}.{method_name}",
-                            route=method_get_handler,
-                            methods=["get"],
-                        )
-                        app.add_route(
-                            f"/{service_handler.full_name}.{method_name}",
-                            route=request_handler,
-                            methods=["post"],
-                        )
-                        logger.info(f"  processing {method_name}")
-                        method_descriptor: MethodDescriptor = (
-                            service_handler.methods_by_name[method_name]
-                        )
-                        method_options: MethodOptions = method_descriptor.GetOptions()
-                        fields: List[FieldDescriptor] = method_options.ListFields()
-                        for field in fields:
-                            if field[0].full_name == "google.api.http":
-                                http_rule: HttpRule = field[1]
-                                for supported_method in supported_methods:
-                                    http_path = getattr(http_rule, supported_method)
-                                    if http_path != "":
-                                        app.add_route(
-                                            http_path,
-                                            route=request_handler,
-                                            methods=[supported_method],
-                                        )
-        logger.info(f"started gateway on port {http_port}")
-        return await serve(app, config)
-    except Exception as ex:
-        logger.error(str(ex))
+    for descriptor in server.descriptors:
+        for service_name in descriptor.services_by_name:
+            service_handler: ServiceDescriptor = descriptor.services_by_name[
+                service_name
+            ]
+            if service_handler.full_name.startswith("grpc."):
+                logger.info(f"skipping service {service_handler.name}")
+            else:
+                logger.info(f"processing service {service_handler.name}")
+                for method_name in service_handler.methods_by_name.keys():
+                    method_get_handler, request_handler = get_http_handlers(
+                        descriptor, service_handler.name, method_name, channel
+                    )
+                    service_name = method_name[1:].split("/")[0].split(".")[-1]
+                    app.add_route(
+                        f"/{service_handler.full_name}.{method_name}",
+                        route=method_get_handler,
+                        methods=["get"],
+                    )
+                    app.add_route(
+                        f"/{service_handler.full_name}.{method_name}",
+                        route=request_handler,
+                        methods=["post"],
+                    )
+                    logger.info(f"  processing {method_name}")
+                    method_descriptor: MethodDescriptor = (
+                        service_handler.methods_by_name[method_name]
+                    )
+                    method_options: MethodOptions = method_descriptor.GetOptions()
+                    fields: List[FieldDescriptor] = method_options.ListFields()
+                    for field in fields:
+                        if field[0].full_name == "google.api.http":
+                            http_rule: HttpRule = field[1]
+                            for supported_method in supported_methods:
+                                http_path = getattr(http_rule, supported_method)
+                                if http_path != "":
+                                    app.add_route(
+                                        http_path,
+                                        route=request_handler,
+                                        methods=[supported_method],
+                                    )
+
+    if not http_port:
+        http_port = find_free_port(7070)
+
+    logger.info(f"starting gateway on port {http_port}")
+    config = Config()
+    config.bind = [f"0.0.0.0:{http_port}"]
+
+    return asyncio.get_event_loop().create_task(serve(app, config))
